﻿########################################################
##
##  Hardware Manufacturers and Models from the WMI
##  get written into Task Sequence variables
##
##  Author: Andreas Kupferschmied/Netree
##  Date:   12.03.2013
##
##  History
##         001: First Version
##
########################################################

#Read the Path from which the Script is running
function GetScriptPath ()
{
  $Invocation = (Get-Variable MyInvocation -Scope 1).Value
  Split-Path $Invocation.MyCommand.Path
}

#The XML File needs to be opened and written into a variable
function OpenXML ()
{
  try
  {
    #Read in mappings.xml
    [xml]$mappings = get-content "$ScriptPath\mappings.xml"
    return $mappings
  }
  catch
  {
    #This error occurs, if the XML file has unclosed tags or similar defects in the code
    WriteLog "ERROR: XML could not be loaded! Wrong format in XML File!"
    Exit 99999
  }
}

#Function to write information for debugging into logfiles
#The logfile is in the mappings folder on the server
function WriteLog ([string]$Line)
{
  $LineToWrite = $tsenv.Value("_SMSTSMachineName") + " " + $DateTime + ": " + $Line
  Out-File -InputObject $LineToWrite -FilePath $ScriptPath\TSVariables.log -Append -ErrorAction SilentlyContinue
  Write-Host $LineToWrite
}

#The task sequence environment variable is used to read and write task sequence variables
$tsenv = New-Object -COMObject Microsoft.SMS.TSEnvironment
$DateTime = Get-Date -format G
$WriteBackWMIValues = $True

#The path to the script
$ScriptPath = GetScriptPath

#For debugging or the application of BIOS settings the version gets stored in a variable
$BIOS = Get-WmiObject Win32_Bios
$tsValBiosVersion = $BIOS.SMBIOSBIOSVersion
WriteLog "BIOS Version = $tsValBiosVersion" 

#The necessary information (manufacturer, model) is stored in the Win32_ComputerSystem container of the WMI
$System = Get-WmiObject Win32_ComputerSystem

$tsValComputerType = "physical"

#Open mappings.xml File
$HWMapping = OpenXML

#The values in the XML file get selected through the parent nodes
#Above the manufacturer node (with the actual manufacturer name for the mapping) are two nodes, mappings and manufacturers
$MappedValuesManufacturers = $HWMapping.mappings.manufacturers
$MappedValuesModels = $HWMapping.mappings.models
$IsFoundManufacturer = $False
$IsFoundModel = $False

#The loop searches the manufacturers nodes in the xml for a value that matches what the script read from the WMI
#The matching string is in the Parentnode "SearchString" and the Value for the TS variable in the "manufacturer" node
foreach($MappedValue in $MappedValuesManufacturers.mapping){
  If($System.Manufacturer -match $MappedValue.searchString)
  { 
    #If the node is found, the flag is set, so that the output for debugging at the end of the script does not run
    $IsFoundManufacturer = $True
    #In the XML file more can be stored than just the manufacturer under each manufacturers-SearchString
    #Additional values get stored into new TS variables with the name of the tag they are put in as variablename
    foreach($Values in $MappedValue.ChildNodes)
    {
      $tsenv.Value($Values.Name) = $Values.InnerText
      If($WriteBackWMIValues = $True)
      {
        #The TS variables and the mapped values get written to the logfile
        WriteLog ($Values.Name + " = '" + $Values.InnerText+"'")
      }
    }
  }
}

#The loop searches the models nodes in the xml for a value that matches what the script read from the WMI
#The matching string is in the Parentnode "SearchString" and the Value for the TS variable in the "model" node
foreach($MappedValue in $MappedValuesModels.mapping){
  If($System.Model -match $MappedValue.searchString)
  { 
    #If the node is found, the flag is set, so that the output for debugging at the end of the script does not run
    $IsFoundModel = $True
    #In the XML file more can be stored than just the model under each models-SearchString
    #Additional values get stored into new TS variables with the name of the tag they are put in
    foreach($Values in $MappedValue.ChildNodes)
    {
      $tsenv.Value($Values.Name) = $Values.InnerText
      If($WriteBackWMIValues = $True)
      {
        #The TS variables and the mapped values get written to the logfile
        WriteLog ($Values.Name + " = '" + $Values.InnerText+"'")
      }
    }
  }
}

#If the manufacturer is not found in any of the SearchString-Nodes, the value from the WMI gets written to the logfile for debugging
If($IsFoundManufacturer -eq $False)
{
  WriteLog ("ERROR in Mapping, Manufacturer in mappings.xml not Found!")
  WriteLog ("The following Manufacturer was found in WMI: '$($System.Manufacturer)'")
}

#If the model is not found in any of the SearchString-Nodes, the value from the WMI gets written to the logfile for debugging
If($IsFoundModel -eq $False)
{
  WriteLog ("ERROR in Mapping, Model in mappings.xml not Found!")
  WriteLog ("The following Model was found in WMI: '$($System.Model)'")
}
